from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Rectangle as Shape
from rocolib.api.ports import AnchorPort
from rocolib.utils.utils import decorateGraph
from rocolib.utils.dimensions import getDim
from rocolib.utils.transforms import Translate, RotateZ
from rocolib.utils.numsym import dot


class ServoMotor(FoldedComponent):
  def define(self):
    self.addParameter("angle", 0, paramType="angle", minValue=None, maxValue=None)
    self.addParameter("servo", "fs90r", paramType="dimension")
    self.addInterface("mount", AnchorPort(self, self.getGraph(), "horn", Translate([0,0,0])))
    self.addFaceInterface("horn", "horn")

  def assemble(self):
    s = self.getParameter("servo")
    a = self.getParameter("angle")
    dz = getDim(s, "hornheight")
    dy = getDim(s, "motorlength") / 2 - getDim(s, "hornoffset")

    f = Shape("horn", 0, 0)
    decorateGraph(f, Shape("hole", 1, 1))
    self.addFace(f)
    self.setInterface("mount", AnchorPort(self, self.getGraph(), "horn", dot(RotateZ(a), Translate([0,-dy,dz]))))

if __name__ == "__main__":
    ServoMotor.test()
