from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face, Rectangle
import rocolib.utils.numsym as np

class UChannel(FoldedComponent):
  def define(self):
    self.addParameter("length", 100, paramType="length")
    self.addParameter("width", 50, paramType="length")
    self.addParameter("depth", 20, paramType="length")

    # Minimum of 45deg to make sure the geometry stays convex
    self.addParameter("angle", optional=True, overrides=("tangle", "bangle"))
    self.addParameter("tangle", 80, paramType="angle", minValue=45)
    self.addParameter("bangle", 135, paramType="angle", minValue=45)

    for i in range(3):
      self.addEdgeInterface("topedge%d" % i, "r%d.e0" % i, ["depth", "width"][i % 2])
      self.addEdgeInterface("botedge%d" % i, "r%d.e2" % i, ["depth", "width"][i % 2])
      self.addFaceInterface("face%d" % i, "r%d" % i)
    self.addEdgeInterface("top", ["r%d.e0" % i for i in range(3)], ["depth", "width", "depth"])
    self.addEdgeInterface("bot", ["r%d.e2" % (2-i) for i in range(3)], ["depth", "width", "depth"])

    self.addEdgeInterface("ledge", "r0.e3", "length")
    self.addEdgeInterface("redge", "r2.e1", "length")

  def modifyParameters(self):
    if self.getParameter("angle") is not None:
      self.setParameter("bangle", self.getParameter("angle"))
      self.setParameter("tangle", self.getParameter("angle"))

  def assemble(self):
    bangle = 90 - self.getParameter("bangle")
    tangle = 90 - self.getParameter("tangle")

    length = self.getParameter("length")
    width = self.getParameter("width")
    depth = self.getParameter("depth")

    def dl(a):
        return np.tan(np.deg2rad(a)) * depth

    rs = []
    rs.append(Face("", (
      (depth, dl(tangle)),
      (depth, length - dl(bangle)),
      (0, length), 
      (0,0)
    )))
    rs.append(Rectangle("", width, length - dl(tangle) - dl(bangle)))
    rs.append(Face("", (
      (0, length), 
      (0,0),
      (depth, dl(bangle)),
      (depth, length - dl(tangle))
    )))

    fromEdge = None
    for i in range(3):
      self.attachEdge(fromEdge, rs[i], "e3", prefix="r%d"%i, angle=90, root=(i==1))
      fromEdge = 'r%d.e1' % i

if __name__ == "__main__":
  UChannel.test()

  #test transform3D
  r = UChannel()
  r.makeOutput(transform3D=[[1,0,0,0],[0,-1,0,0],[0,0,-1,0],[0,0,0,1]], default=False)
  g = r.getGraph()
  for f in g.faces:
    print ("########", f.name)
    print(f.get3DCoords())

  #test sympy
  r = UChannel()
  r.setParameter("angle", 90)
  r.makeOutput(useDefaultParameters=False, default=False)
  g = r.getGraph()
  for f in g.faces:
    print ("########", f.name)
    #print(f.transform2D)
    #print(f.transform3D)
    #print(f.get2DCoords())
    np.pprint(f.get3DCoords())
